var projectExtensions={
	
	/**
	 * Insert tabs before (to the left of) the default tabs in the project view page.
	 * This function is called automatically from the base project view page. You just need to add your tabs.
	 */
	tabsBefore:function(tabSet){
		/* Example of a tab. See the tab fucntion in ui.js for more details.
		tabSet.tab({ 
			id:'before', 
			label:'Before', 
			url:'/api/project/'+data['id']+'/permission', 
			successHandler:function(){ $("before").next().innerHTML="Tab before default project tabs"; }
		 });
		*/
		tabSet.tab({ 
			id:'plates', 
			label:'Plates', 
			url:'/api/project/'+data['id']+'/plate?sortby=id&sortdescending=yes', 
			headers:['Barcode','Owner','Description','Best score'],
			sortOrders:['name','user.fullname','','crystalscore.scoreindex'],
			cellTemplates:['<a href="/plate/{{id}}">{{name}}</a>','<a href="/user/{{ownerid}}">{{ownername}}</a>',[projectExtensions.trimDescription,'description'],[Plate.renderPlateBestScoreCell,'bestscorelabel'] ]
		 });
		tabSet.tab({ 
			id:'proteins', 
			label:'Proteins', 
			url:'/api/project/'+data['id']+'/protein', 
			successHandler:function(transport){ projectExtensions.writeProteins(transport.responseJSON.rows);  },
			failureHandler:function(transport){ $("proteins_body").innerHTML=""; projectExtensions.writeProteinCreateForm();  }
		 });
		tabSet.tab({ 
			id:'crystals', 
			label:'Crystals', 
			url:'/api/crystal/projectid/'+data['id']+'?sortby=id&sortdescending=yes', 
			headers:['Image','Plate','Drop and number'],
			cellTemplates:[ [Crystal.getThumbnailLink,'id'], '<a href="/plate/{{plateid}}">{{platename}}</a>',  [Crystal.getTextLink,'id'] ],
		 });
	},
	
	/**
	 * Insert tabs after (to the right of) the default tabs in the project view page.
	 * This function is called automatically from the base project view page. You just need to add your tabs.
	 */
	tabsAfter:function(tabSet){
		/* Example of a tab. See the tab function in ui.js for more details.
		tabSet.tab({ 
			id:'after', 
			label:'After', 
			url:'/api/project/'+data['id']+'/permission', 
			successHandler:function(){ $("after").next().innerHTML="Tab after default project tabs"; }
		 });
		*/
	},
	
	trimDescription:function(item){
		var maxLength=50;
		var desc=item.description;
		if(desc.length<=maxLength){ return desc; }
		desc=desc.substr(0,maxLength-3);
		var lastSpace=desc.lastIndexOf(" ");
		if(0<lastSpace){
			desc=desc.substr(0,lastSpace);
		}
		return desc+'...';
	},

	
	writeProteins:function(rows, proteinIdToOpen, constructIdToOpen){
		var tab=$("proteins_body");
		tab.innerHTML='';
		rows.each(function(r){
			var ti=tab.treeItem({
				record:r,
				id:"protein"+r.id,
				header:r.name+" ("+r.proteinacronym+")",
				updater:projectExtensions.renderProtein
			});
			if(r.id==proteinIdToOpen){
				ui.doToggleTreeItem($('protein'+r.id));
			}
		});
		projectExtensions.writeProteinCreateForm();
	},
	renderProtein:function(treeHeader,constructIdToOpen){
		var ti=treeHeader.up(".treeitem")
		var tb=ti.down(".treebody");
		tb.innerHTML='';
		var prot=ti.record;
		var f=ti.form({
			action:"/api/protein/"+prot['id'],
			method:"patch"
		});
		f.hiddenField('projectid',data['id']);
		if(canEdit){
			f.textField({
				label:'Name',
				name:'name',
				value:prot['name'],
				helpText:'Protein name must be unique within this project',
			});
			f.textField({
				label:'Protein acronym',
				name:'proteinacronym',
				value:prot['proteinacronym'],
				helpText:'Short code for synchrotron safety info',
			});
		}
		f.textField({
			readonly:!canEdit,
			label:'Description',
			name:'description',
			value:prot['description'],
			helpText:'A short description of the protein',
			readOnly:!canEdit
		});
		new Ajax.Request('/api/protein/'+prot['id']+'/construct',{
			method:"get",
			onSuccess:function(transport){ projectExtensions.writeConstructs(transport,constructIdToOpen); },
			onFailure:function(transport){ projectExtensions.writeConstructCreateForm(tb); },
		});
	},
	writeProteinCreateForm:function(){
		var tab=$("proteins_body");
		if(canEdit){
			var createitem=tab.treeItem({
				header:'Create a new protein'
			});
			f=createitem.form({
				action:"/api/protein",
				method:"post"
			});
			f.hiddenField('projectid',data['id']);
			f.textField({
				label:'Name',
				name:'name',
				helpText:'Protein name must be unique within this project'
			});
			f.textField({
				label:'Protein acronym',
				name:'proteinacronym',
				helpText:'Short code for synchrotron safety info'
			});
			f.textField({
				label:'Description',
				name:'description',
				helpText:'A short description of the protein'
			});
			f.createButton({
				afterSuccess:function(createResponse){
					new Ajax.Request('/api/project/'+data.id+'/protein',{
						method:'get',
						onSuccess:function(transport){
							projectExtensions.writeProteins(transport.responseJSON.rows, createResponse.created.id);
						}
					})
				}
			});
			f.up(".treeitem").addClassName("noprint");
		}
	},
	
	
	writeConstructs:function(transport,constructIdToOpen){
		var wrapper;
		transport.responseJSON.rows.each(function(r){
			wrapper=$("protein"+r['proteinid']+"_body");
			wrapper.treeItem({
				record:r,
				id:"construct"+r.id,
				header:"Construct: "+r.name,
				updater:projectExtensions.renderConstruct
			});
			if(r.id==constructIdToOpen){
				ui.doToggleTreeItem($("construct"+r.id));
			}
		});
		projectExtensions.writeConstructCreateForm(wrapper);
	},
	renderConstruct:function(treeHeader){
		var ti=treeHeader.up(".treeitem")
		var tb=ti.down(".treebody");
		tb.innerHTML='';
		var con=ti.record;
		var f=ti.form({
			action:"/api/construct/"+con['id'],
			method:"patch"
		});
		f.hiddenField('constructid',con['id']);
		if(canEdit){
			f.textField({
				label:'Name',
				name:'name',
				value:con['name'],
				helpText:'Construct name must be unique within this protein',
			});
		}
		f.textField({
			readonly:!canEdit,
			label:'Description',
			name:'description',
			value:con['description'],
			helpText:'A short description of the construct',
			readOnly:!canEdit
		});
		projectExtensions.getSequences(con.id);
	},
	writeConstructCreateForm:function(wrapper){
		if(canEdit){
			var createitem=wrapper.treeItem({
				header:'Create a new construct'
			});
			f=createitem.form({
				action:"/api/construct",
				method:"post"
			});
			var prot=wrapper.up(".treeitem").record;
			f.hiddenField('projectid',data['id']); 
			f.hiddenField('proteinid',prot['id']); 
			f.textField({
				label:'Name',
				name:'name',
				helpText:'Construct name must be unique within this protein'
			});
			f.textField({
				label:'Description',
				name:'description',
				helpText:'A short description of the construct'
			});
			f.createButton({
				afterSuccess:function(createResponse){
					new Ajax.Request('/api/project/'+data.id+'/protein',{
						method:'get',
						onSuccess:function(transport){
							projectExtensions.writeProteins(transport.responseJSON.rows, prot['id'], createResponse.created.id);
						}
					})
				}
			});
		}
	},
	
	getSequences:function(constructId){
		new Ajax.Request('/api/construct/'+constructId+'/sequence',{
			method:'get',
			onSuccess:function(transport){ projectExtensions.getSequences_onSuccess(transport, constructId); },
			onFailure:function(transport){ projectExtensions.getSequences_onFailure(transport, constructId); },
		});
	},
	getSequences_onSuccess:function(transport, constructId){
		if(!AjaxUtils.checkResponse(transport)){ return false; }
		projectExtensions.renderSequences(transport.responseJSON.rows,constructId);
	},
	getSequences_onFailure:function(transport, constructId){
		if(404==transport.status){
			projectExtensions.renderSequences([],constructId);
		} else {
			return AjaxUtils.checkResponse(transport);
		}
	},
	renderSequences:function(sequences, constructId){
		sequences.each(function(seq){
			var f=projectExtensions.generateSequenceBlock(seq);
			$("construct"+seq.constructid+"_body").appendChild(f);
		});
		if(canEdit){
			f=$("construct"+constructId+"_body").form({
				action:'/api/sequence/',
				method:'post',
				id:"construct"+constructId+"addseqform"
			});
			ff=f.formField({
				label:'<input type="button" onclick="projectExtensions.showSequenceWindow(this)" id="construct'+constructId+'addseqbuttton" value="Add sequence..." />',
				name:'',
				value:''
			});
			ff.innerHTML+='&nbsp;';
			ff.sequence={};
			ff.down("input").dataset.constructid=constructId;
		}
	},

	generateSequenceBlock:function(seq){
		var f=ui.form({
			action:'/api/sequence/'+seq.id,
			method:'patch',
			id:'seq'+seq.id
		});
		var seqToShow=projectExtensions.generateSequencePreviewString(seq);
		var ff=f.formField({
			label:'Sequence: '+seq.name,
			name:'seq'+seq.id,
			value:seq.name
		});
		ff.sequence=seq;
		var viewButton='<input type="button" onclick="projectExtensions.showSequenceWindow(this)" value="View..." /> ';
		var deleteButton="";
		if(canEdit){
			viewButton=viewButton.replace("View","View/Edit");
			deleteButton='<input type="button" onclick="projectExtensions.deleteSequence(this)" value="Delete" />';
		}
		ff.innerHTML+=('<span class="seqpreview">'+seqToShow+"</span><br/>"+viewButton+deleteButton);
		ff.down("input").dataset.constructid=seq.constructid;
		return f;
	},
	
	sequencePreviewLength:36,

	generateSequencePreviewString:function(seq){
		var seqToShow=seq.dnasequence;
		if(""!=seq.proteinsequence){ seqToShow=seq.proteinsequence; }
		if(""==seqToShow){ return "(no DNA or protein sequence)"; }
		if(seqToShow.length>projectExtensions.sequencePreviewLength){
			return seqToShow.substr(0,projectExtensions.sequencePreviewLength).toUpperCase()+"...";
		}
		return seqToShow.toUpperCase();
	},
	
	showSequenceWindow:function(btn){
		var seq=btn.up("label").sequence;
		var seqName=seq.name||"";
		var dnaSeq=seq.dnasequence||"";
		var protSeq=seq.proteinsequence||"";
		var formAction=seq.id ? '/api/sequence/'+seq.id : '/api/sequence/'; 
		var formMethod=seq.id ? 'patch' : 'post'; 
		var formClasses=seq.id ? '' : 'create'; 
		var boxTitle=seq.name ? 'Sequence: '+seq.name : 'Add a sequence'; 
		var buttonLabel=seq.id ? 'Save changes' : 'Add sequence';
		var buttonAction=projectExtensions.submitSequenceForm;
		var mb=ui.modalBox({ title:boxTitle, content:'', confirmClose:true });
		var f=modalBox.form({
			id:'sequenceform',
			action:formAction,
			method:formMethod,
		});
		f.hiddenField("seqconstructid",btn.dataset.constructid);
		f.textField({
			label:'Name',
			name:'seqname',
			helpText:'Sequence name must be unique for this construct',
			value:seqName,
			readonly:!canEdit
		});
		if(canEdit || (!canEdit && ""!=dnaSeq)){
			f.textArea({
				label:'DNA sequence, if known',
				name:'dnasequence',
				helpText:'The DNA sequence',
				value:dnaSeq,
				readonly:!canEdit
			});
		}
		if(canEdit || (!canEdit && ""!=protSeq)){
			f.textArea({
				label:'Protein sequence',
				name:'proteinsequence',
				helpText:'The protein sequence',
				value:protSeq,
				readonly:!canEdit
			});
		}
		if(canEdit){
			f.buttonField({
				id:"seqsubmit",
				label:buttonLabel,
				onclick:buttonAction
			});
			fieldValidations['seqname']="required";
			fieldValidations['dnasequence']="dnaSequence";
			fieldValidations['proteinsequence']="proteinSequence";
			$$("#modalBox textarea, #modalBox input[type=text]").each(function(elem){ 
				elem.onkeyup=function(){ $("modalBox").tainted=true; }; 
				elem.style.width="70%"; 
			});
			$$("#modalBox textarea").each(function(elem){ 
				elem.style.height="10em"; 
				elem.onclick=function(){ $("modalBox").tainted=true; }; 
			});
			$("proteinsequence").observe("keyup",projectExtensions.onEditProteinSequence);
			$("proteinsequence").observe("click",projectExtensions.onEditProteinSequence);
			$("dnasequence").observe("keyup",projectExtensions.onEditDnaSequence);
			$("dnasequence").observe("click",projectExtensions.onEditDnaSequence);
		}
	},

	onEditProteinSequence:function(){
		var seq=$("proteinsequence").value+"";
		seq=seq.replace(/[^\*ACDEFGHIKLMNPQRSTVWY]/gi, "").toUpperCase();
		if(""==seq){ return; }
		$("proteinsequence").value=Protein.formatProteinSequence(seq);
	},
		
	onEditDnaSequence:function(){
		var seq=$("dnasequence").value+"";
		seq=seq.replace(/[^\*ACGT]/gi, "").toUpperCase();
		if(""==seq){ return; }
		$("dnasequence").value=Protein.formatDnaSequence(seq);
	},

	dnaMatchesProtein:function(){
		var dna=$("dnasequence").value.replace(/\s/g,"");
		var prot=$("proteinsequence").value.replace(/\s/g,"");
		if(""==prot || ""==dna){ return true; }
		if(prot.length*3 != dna.length){ return false; }
		var convertedDna=Protein.dnaToProtein(dna);
		if(convertedDna===prot){ return true; }
		return false;
	},
	
	submitSequenceForm:function(){
		var isValid=validator.validate($('seqname'));
		isValid=validator.validate($('proteinsequence')) && isValid;
		isValid=validator.validate($('dnasequence')) && isValid; 
		if(!isValid){ return false;	}
		if(!projectExtensions.dnaMatchesProtein()){
			if(!confirm("DNA and protein sequences do not match.\n\nClick OK to generate the protein sequence from the DNA sequence.\nClick Cancel to edit by hand.")){
				return false;
			}
			$("proteinsequence").value=Protein.dnaToProtein($("dnasequence").value);
		} else if(""==$('proteinsequence').value && ""!=$('dnasequence').value){
			if(confirm("The protein sequence is empty.\n\nClick OK to generate the protein sequence from the DNA sequence.\nClick Cancel to leave it empty.")){
				$("proteinsequence").value=Protein.formatProteinSequence(Protein.dnaToProtein($("dnasequence").value));
			}
		}
		$("seqsubmit").up("label").addClassName("updating");
		var method="post";
		if("post"!==$("sequenceform").dataset.ajaxmethod){ method="patch"; }
		new Ajax.Request($("sequenceform").action,{
			method:method,
			parameters:{
				csrfToken:csrfToken,
				name:$("seqname").value,
				dnasequence:$("dnasequence").value,
				proteinsequence:$("proteinsequence").value,
				constructid:$("seqconstructid").value,
			},
			onSuccess:projectExtensions.submitSequenceForm_onSuccess,
			onFailure:projectExtensions.submitSequenceForm_onFailure,
		})
		
	},
	submitSequenceForm_onSuccess:function(transport){
		$("seqsubmit").up("label").removeClassName("updating");
		if(!AjaxUtils.checkResponse(transport)){ return false; }
		var updated=transport.responseJSON.updated;
		var created=transport.responseJSON.created;
		if(updated){
			var seqBox=$("seq"+updated.id);
			seqBox.down(".label").innerHTML="Sequence: "+updated.name;
			seqBox.down(".seqpreview").innerHTML=projectExtensions.generateSequencePreviewString(updated);
			seqBox.down("label").sequence=updated;
		} else if(created){
			var f=projectExtensions.generateSequenceBlock(created);
			$("construct"+created.constructid+"addseqform").insert({before:f});	
		}
		$("modalBox").tainted=false;
		ui.closeModalBox();
	},
	submitSequenceForm_onFailure:function(){
		$("seqsubmit").up("label").removeClassName("updating");
		AjaxUtils.checkResponse(transport);
	},

	deleteSequence:function(elem){
		if(!canEdit){ 
			alert("You don't have permission to delete sequences");
			return false;
		}
		var frm=$(elem).up("form");
		elem.up("label").addClassName("updating");
		window.setTimeout(function(){
			new Ajax.Request(frm.action, {
				method:'delete',
				parameters:{ csrfToken:csrfToken },
				onSuccess:projectExtensions.deleteSequence_onSuccess,
				onFailure:projectExtensions.deleteSequence_onFailure
			});
		},250); //give the "updating" time to show - deletion is very quick, may be jarring
	},
	deleteSequence_onSuccess:function(transport){
		if(!AjaxUtils.checkResponse(transport)){ return false; }
		var deletedId=transport.responseJSON.deleted;
		$("seq"+deletedId).remove();
	},
	deleteSequence_onFailure:function(transport){
		$("seqsubmit").up("label").removeClassName("updating");
		AjaxUtils.checkResponse(transport);
		
	},
	
}